[<Microsoft.FSharp.Core.AutoOpen>]
module Degenz.Types

open System
open Degenz

[<Measure>]
type mins

[<Measure>]
type GBT

type ItemId =
    | Virus = 0
    | RemoteAccess = 1
    | Worm = 2
    | Firewall = 6
    | Encryption = 7
    | Cypher = 8

type StatId =
    | Strength = 0
    | Focus = 1
    | Luck = 2
    | Charisma = 3

type StatConfig = {
    Id : StatId
    BaseDecayRate : float
    BaseMinMax : Range
}

type PlayerStat = {
    Id : StatId
    Amount : int
    ModMinMax : Range
    LastRead : DateTime
}
with static member empty = { Id = StatId.Strength ; Amount = 0 ; ModMinMax = Range(0, 100) ; LastRead = DateTime.UtcNow}

type Stats = {
    Strength : PlayerStat
    Focus : PlayerStat
    Luck : PlayerStat
    Charisma : PlayerStat
}
with static member empty = { Strength = PlayerStat.empty ; Focus = PlayerStat.empty ; Luck = PlayerStat.empty ; Charisma = PlayerStat.empty }

type HackResult =
    | Strong
    | Weak

type DiscordPlayer = { Id: uint64; Name: string }
    with static member empty = { Id = 0uL ; Name = "None" }

type HackEvent = {
    IsInstigator : bool
    Adversary : DiscordPlayer
    Success : bool
    HackId : int
}

type PlayerEventType =
    | Hacking of HackEvent
    | Shielding of shieldId : int
    | Stealing of instigator : bool * adversary : DiscordPlayer
    | Imprison

type PlayerEvent =
    { Type : PlayerEventType
      Cooldown : int<mins>
      Timestamp : DateTime }

type HackItem = {
    Id : int
    Name : string
    Price : int<GBT>
    Power : int
    Class : int
    Cooldown : int<mins>
}

type ShieldItem = {
    Id : int
    Name : string
    Price : int<GBT>
    Class : int
    Cooldown : int<mins>
}

type FoodItem = {
    Id : int
    Name : string
    Price : int<GBT>
    TargetStat : StatId
    BoostAmount : int
}

type AccessoryItem = {
    Id : int
    Name : string
    Price : int<GBT>
    TargetStat : StatId
    PassiveBoost : int
}

type ItemType =
    | Hack of power : int * hackClass : int * cooldown : int<mins>
    | Shield of shieldClass : int * cooldown : int<mins>
    | Food of targetStat : StatId * boostAmount : int
    | Accessory of targetStat : StatId * passiveBoost : int
and Item = {
    Id : int
    Name : string
    Price : int<GBT>
    Type : ItemType
}
and Inventory = Item list
and PlayerData = {
    DiscordId : uint64
    Name : string
    Inventory : Inventory
    Events : PlayerEvent array
    Stats : Stats
    Bank : int<GBT>
}
//        Achievements : string array
//        XP : int
with member this.basicPlayer = { Id = this.DiscordId ; Name = this.Name }
     static member empty =
         { DiscordId = 0uL
           Name = "None"
           Inventory = []
           Events = [||]
           Stats = Stats.empty
//               Achievements = [||]
//               XP = 0
           Bank = 0<GBT> }
